import { csv } from 'd3';
import {
  select,
  json,
  pie,
  arc,
  scaleOrdinal,
  format,
} from 'd3';

// Colors for the sections in Pie Chart
const sectionColors = [
  'red',
  'purple',
  'yellow',
  'lime',
  'orange',
  'skyblue',
  'crimson',
  'wheat',
  'turquoise',
  'ligthblue',
];

// Data set URI
const csvUrl =
  'https://gist.githubusercontent.com/pavankalyan1507/979c4d71de6cb8c310309d0ef2803228/raw/6c6352eae64cf8ea730c0a317a678557a066eb54/population';

// Function to fetch data from the URI
const fetchData = async () => {
  const data = await csv(csvUrl);
  return data;
};

const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');

// Format axes text
const xAxisTickFormat = (number) =>
  format('.4~g')(number);

// Render Pie Chart on the canvas
const render = (data, total) => {
  const pieData = pie().value(
    (d) => d.Population
  )(data);
  const colors = scaleOrdinal()
    .domain(data)
    .range(sectionColors);

  // Object for generating Arcs of the sections
  const arcGenerator = d3
    .arc()
    .innerRadius(0)
    .outerRadius(250);

  const segments = arc()
    .innerRadius(0)
    .outerRadius(175)
    .padAngle(0)
    .padRadius(0);

  const sections = svg
    .append('g')
    .attr('transform', `translate(250,250)`)
    .selectAll('path')
    .data(pieData);

  sections
    .enter()
    .append('path')
    .attr('d', segments)
    .attr('fill', (d) =>
      colors(d.data.Population)
    );

  // Render text on each section
  sections
    .enter()
    .append('text')
    .text(
      (d) =>
        xAxisTickFormat(
          d.data.Population / 1000
        ) + 'K'
    )
    .attr('transform', function (d) {
      return (
        'translate(' +
        arcGenerator.centroid(d) +
        ')'
      );
    })
    .style('text-anchor', 'middle')
    .style('fill', 'black')
    .style('font-size', 15);

  // Render legend for the data

  const legends = svg
    .append('g')
    .attr('transform', 'translate(500,100)')
    .selectAll('.legends')
    .data(pieData);

  const legend = legends
    .enter()
    .append('g')
    .classed('.legends', true)
    .attr('transform', (d, i) => {
      return `translate(0,${(i + 1) * 30})`;
    });

  legend
    .append('rect')
    .attr('width', 20)
    .attr('height', 20)
    .attr('fill', (d) =>
      colors(d.data.Population)
    );
  legend
    .append('text')
    .attr('x', 25)
    .attr('y', 15)
    .attr('class', 'legend_text')
    .text((d) => d.data.Countries);
  legend
    .append('text')
    .attr('x', 250)
    .attr('y', 15)
    .attr('class', 'legend_value')
    .text(
      (d) =>
        xAxisTickFormat(
          d.data.Population / 1000
        ) + 'K'
    );
};

// Main function
const run = async () => {
  let data = await fetchData();
  let total = 0;
  data.forEach((d) => {
    d.Population = +d.Population;
    total += d.Population;
  });
  render(data, total);
};
run();
